% Generate synthetic dataset and note corresponding likelihood
randn('state', 0);
rand('state', 0);
D = 3;
ww = 5*randn(D, 1);
N = 20;
X = [randn(N, D-1), ones(N, 1)];
yy = rand(N, 1) < (1./(1 + exp(-X*ww)));
neg_log_like_grad = @(w) logreg_negLlike(w, X, yy);

% If you rerun the fitting with different seeds you'll see we don't
% get quite the same answer each time. I'd need to set the learning rate
% schedule better, and/or run for longer, to get better convergence.
randn('state', 2);
rand('state', 2);

% Simple stochastic steepest descent with decreasing learning rate.
% Here each update includes the whole dataset because the dataset is
% tiny. However, we still have stochastic updates, as each update
% uses a different random weight drawn from the current variational
% approximation to the posterior.
Lsigma_w = log(10); % Initialize prior width broader than it actually was. (We'll learn to fix that.)
mm = zeros(D, 1);
LL = zeros(D);
LL(1:(D+1):end) = Lsigma_w;
eta0 = 0.1;
tau = 0.1;
for ii = 1:10000
    eta = eta0/(1 + tau*eta0*ii);
    [J, mm_bar, LL_bar, Lsigma_w_bar] = svi_grad(mm, LL, Lsigma_w, neg_log_like_grad);
    mm = mm - eta*mm_bar;
    LL = LL - eta*LL_bar;
    Lsigma_w = Lsigma_w - eta*Lsigma_w_bar;
    % hack to stop extreme settings of hyperparameters early in training:
    Lsigma_w = min(100.0, Lsigma_w)
end

% Extract covariance of the variational posterior from its
% unconstrained parameterization.
L = LL;
L(1:(D+1):end) = exp(LL(1:(D+1):end));
V = L*L';

% Plot data:
clf; hold on;
plot(X(yy==1, 1), X(yy==1, 2), 'bx');
plot(X(yy==0, 1), X(yy==0, 2), 'ro');
legend({'y=1', 'y=0'});

% Overlay contour plot of approximate predictive distribution:
x_grid = -3:0.05:3;
[X1, X2] = meshgrid(x_grid, x_grid);
X_test = [X1(:), X2(:) ones(numel(X1),1)];
kappa = 1 ./ sqrt(1 + (pi/8)*sum((X_test*V).*X_test, 2));
p_test = 1./(1+exp(-(X_test*mm).*kappa));
P = reshape(p_test, size(X1));
[C, hh] = contour(X1, X2, P, [0.1,0.25,0.5,0.75,0.9]);
clabel(C, hh, 'LabelSpacing', 2000);
xlabel('x_1');
ylabel('x_2');
title('Contours of p(y=1|x,D)');

